import numpy as np
import matplotlib.pyplot as plt
import os

import util

import kernel_regression

def max_score():
    return 2

def timeout():
    return 60

def test():

    figures_directory = 'figures'

    os.makedirs(figures_directory, exist_ok=True)

    x = np.zeros((2, 1))
    z = np.zeros((2, 1))
    for width in [1.0, 0.5, 2]:
        for z_tuple in [(0, 0), (10, 10), (-2, 2)]:
            z[0, 0] = z_tuple[0]
            z[1, 0] = z_tuple[1]

            # Solution values
            expected_output_values = {}
            expected_output_values[(z_tuple[0] + 0, z_tuple[1] + 0)] = 1
            expected_output_values[(z_tuple[0] + 0, z_tuple[1] - 2*width/2)] = 0
            expected_output_values[(z_tuple[0] - 1*width/2, z_tuple[1] - 1*width/2)] = 0
            expected_output_values[(z_tuple[0] - 0.5*width/2, z_tuple[1] + 0.5*width/2)] = 1

            for x_tuple, expected_output in expected_output_values.items():
                x[0, 0] = x_tuple[0]
                x[1, 0] = x_tuple[1]

                actual_output = kernel_regression.boxcar_kernel(x, z, width=width)

                if isinstance(actual_output, np.ndarray):
                    if len(actual_output.shape) == 1:
                        actual_output = actual_output[0]
                    else:
                        actual_output = actual_output[0, 0]

                assert abs(actual_output - expected_output) < 0.01 , 'Incorrect kernel value found for width={}, x={}, z={}. Expected {}, found {}'.format(width, x_tuple, z_tuple, expected_output, actual_output)
     
    test_score = max_score()
    test_output = 'PASS\n'

    # Plot surface of kernel at a specific point z

    z_tuple = (2, -3)
    z[0, 0] = z_tuple[0]
    z[1, 0] = z_tuple[1]

    k = lambda X: boxcar_kernel_all(X, z, width)

    filename = '{}/kernel_boxcar_width_{}.png'.format(figures_directory, width)
    title = 'Boxcar kernel, width = {}, z = ({}, {})'.format(width, z[0, 0], z[1, 0])

    point = np.hstack((z.T, np.ones((1,1))))

    util.plot_surface(k, point, x_min=-6, x_max=6, z_min=0, z_max=2,
            title=title, new_figure=True, show_figure=False, save_filename=filename)
    plt.close()

    return test_score, test_output

def boxcar_kernel_all(X, z, width):
    N, M = X.shape

    y = np.zeros((N, 1))
    for n in range(N):
        x_n = np.reshape(X[n], (M, 1))
        
        y[n] = kernel_regression.boxcar_kernel(x_n, z, width=width)

    return y

if __name__ == "__main__":
    test()
