import numpy as np
import matplotlib.pyplot as plt
import os

import util

import kernel_regression

def max_score():
    return 6

def timeout():
    return 300

def test():

    figures_directory = 'figures'

    os.makedirs(figures_directory, exist_ok=True)

    expected_mse_train_values = {}
    expected_mse_val_values = {}

    expected_mse_train_values[(1, 1)] = 0.0150
    expected_mse_val_values[(1, 1)] = 107.7821
    expected_mse_train_values[(1, 2)] = 0.0150
    expected_mse_val_values[(1, 2)] = 107.7821
    expected_mse_train_values[(1, 4)] = 0.0150
    expected_mse_val_values[(1, 4)] = 107.7821
    expected_mse_train_values[(2, 1)] = 0.0147
    expected_mse_val_values[(2, 1)] = 107.7821
    expected_mse_train_values[(2, 2)] = 0.0147
    expected_mse_val_values[(2, 2)] = 107.7821
    expected_mse_train_values[(2, 4)] = 0.0147
    expected_mse_val_values[(2, 4)] = 107.7821
    expected_mse_train_values[(10, 1)] = 0.0092
    expected_mse_val_values[(10, 1)] = 107.7821
    expected_mse_train_values[(10, 2)] = 0.0092
    expected_mse_val_values[(10, 2)] = 107.7821
    expected_mse_train_values[(10, 4)] = 0.0092
    expected_mse_val_values[(10, 4)] = 90.9174
    expected_mse_train_values[(50, 1)] = 0.0191
    expected_mse_val_values[(50, 1)] = 78.4968
    expected_mse_train_values[(50, 2)] = 0.0371
    expected_mse_val_values[(50, 2)] = 56.6518
    expected_mse_train_values[(50, 4)] = 0.0866
    expected_mse_val_values[(50, 4)] = 81.8353
    expected_mse_train_values[(200, 1)] = 0.0219
    expected_mse_val_values[(200, 1)] = 49.0386
    expected_mse_train_values[(200, 2)] = 0.0612
    expected_mse_val_values[(200, 2)] = 1190.0059
    expected_mse_train_values[(200, 4)] = 0.0265
    expected_mse_val_values[(200, 4)] = 245.6318

    X_train, y_train, X_val, y_val = kernel_regression.load_data()

    for N in [1, 2, 10, 50, 200]:
        for width in [1, 2, 4]:
            print('Testing boxcar kernel for N={} width={}'.format(N, width))

            kernel_function = lambda x, z: kernel_regression.boxcar_kernel(x, z, width=width)

            h = lambda X: kernel_regression.predict_kernel_regression(X, X_train[:N], y_train[:N], kernel_function)

            # Compute and check training error
            mse_train = kernel_regression.mse(y_train[:N], h(X_train[:N]))

            print('\tmse_train:  {:.4f}'.format(mse_train))
            expected_mse_train = expected_mse_train_values[(N, width)]
            assert abs(mse_train - expected_mse_train) < 0.01 , 'Incorrect training MSE value found for boxcar kernel N={}, width={}. Expected {}, found {}'.format(N, width, expected_mse_train, mse_train)

            # Compute and check validation error
            mse_val = kernel_regression.mse(y_val, h(X_val))

            print('\tmse_val:    {:.4f}'.format(mse_val))
            expected_mse_val = expected_mse_val_values[(N, width)]
            assert abs(mse_val - expected_mse_val) < 0.01 , 'Incorrect validation MSE value found for boxcar kernel N={}, width={}. Expected {}, found {}'.format(N, width, expected_mse_val, mse_val)

            # Plot training data and predicted surface

            filename = '{}/regression_boxcar_N_{}_width_{}.png'.format(figures_directory, N, width)
            title = 'Boxcar kernel, width = {}, N = {}'.format(width, N)

            points = np.hstack((X_train[:N], y_train[:N]))

            util.plot_surface(h, points, title=title,
                    new_figure=True, show_figure=False, save_filename=filename)
            plt.close()

    test_score = max_score()
    test_output = 'PASS\n'

    return test_score, test_output

if __name__ == "__main__":
    test()
